module LifenFhir
  class Client

    LOGGED_INFO_STATUS = [200, 201]

    def request(mode, url, params = {})
      before_request

      response = faraday_client.send(mode) do |req|
        req.url url

        req.headers['Authorization']      = "Bearer #{bearer}"
        req.headers['Accept']             =  use_and_remove_accept(params)

        if mode == :post
          req.headers['Content-Type']     = "application/json"
        end

        req.body = JSON.generate(params)
      end

      handle_logger(response)
      handle_errors(response, params)

      handle_response(response)
    end

    def post(url, params = {})
      request(:post, url, params)
    end

    def put(url, params = {})
      request(:put, url, params)
    end

    def get(url, params = {})
      request(:get, url, params)
    end

    private

      def handle_errors(response, params)
        if response.status == 500
          raise_error("500 Error, Internal server error", response, params)
        end
      end

      def handle_logger(response)
        if LOGGED_INFO_STATUS.include? response.status
          logger.info "*** [LIFEN] Success on #{response.env.method.upcase} '#{response.env.url}'"
        else
          logger.fatal "*** [LIFEN] #{response.status} Error on #{response.env.method.upcase} '#{response.env.url}'"
        end
      end

      def handle_response(response)
        if response.headers['Content-Type'].match "json"
          JSON.parse response.body
        else
          response.body
        end
      end

      def faraday_client
        @faraday_client ||= Faraday.new(faraday_options) do |faraday|
          faraday.adapter  Faraday.default_adapter  # make requests with Net::HTTP
        end
      end

      def faraday_options
        options = {url: site}

        options[:proxy] = proxy_url if !proxy_url.nil?

        options
      end

      def site
        LifenFhir.configuration.site
      end

      def proxy_url
        LifenFhir.configuration.proxy_url
      end

      def raise_error(message, response, params)
        message += " -- #{response.env.method.upcase} '#{response.env.url}'"

        raise Error.new(message: message, response: response, params: params, client: "app", token: bearer)
      end

      def logger
        LifenFhir.configuration.logger
      end

      def before_request
      end

      def use_and_remove_accept(params)
        return params.delete :accept if params.is_a?(Hash) and params.has_key? :accept

        "application/json"
      end

      def before_request
      end

      def bearer
        raise "A bearer method must be defined"
      end

  end
end
