module LifenFhir
  class CommunicationRequest < Element

    attribute :number_communications, Integer
    attribute :json_payload, Hash

    attribute :sender, Sender
    attribute :recipients, [Recipient]

    attribute :category, Category, default: Category.new
    attribute :medium, [Medium]
    attribute :patient, Patient
    attribute :attachment, Attachment
    attribute :binary, Binary
    attribute :content_string, ContentString

    attribute :communications, [Communication], default: []

    attribute :status, String

    PAYLOAD_ATTRIBUTES = ["sender", "recipient", "payload", "category"]

    def send
      json = application_client.post("fhir/CommunicationRequest", fhir_payload)

      self.uuid = json["id"]
      self.number_communications = Array(json["issue"]).length

      self
    end

    def self.find_by_uuid(uuid)
      json = application_client.get("fhir/CommunicationRequest/#{uuid}")

      communication_request = new
      communication_request.attributes_from_json(json)

      communication_request
    end

    def communications
      if !@communications_loaded
        load_communications
        @communications_loaded = true
      end

      @communications
    end

    def attributes_from_json(json)
      self.json_payload = json

      self.uuid = json["id"]

      self.status = json.fetch("status") { "unknown" }

      if has_payload
        self.binary = Binary.new.attributes_from_json(Array(json["payload"]).first)
      end

      if has_category
        self.category = Category.new.attributes_from_json(Array(json["category"]).first)
      end

      if has_sender
        self.sender = Practitioner.new.attributes_from_json(json["sender"])
      end

      if has_recipient
        self.recipients = Array(json["recipient"]).map do |recipient_json|
           Practitioner.new.attributes_from_json(recipient_json)
        end
      end

    end

    def method_missing(method, *arguments, &block)
      attribute_name = method.to_s.gsub!('has_', '')
      if PAYLOAD_ATTRIBUTES.include? attribute_name
        json_payload.key?(attribute_name)
      else
        super
      end
    end

    private

      def fhir_payload

        payload = {
          resourceType: "CommunicationRequest",
          sender: [ sender.fhir_payload_as_reference ],
          recipient: recipients.map(&:fhir_payload_as_reference),
          contained: [],
          medium: medium.map(&:fhir_payload) ,
          category: [ category.fhir_payload],
          payload: [ document_content ]
        }

        if patient
          payload[:contained] << patient.fhir_payload
          payload[:subject] = [
            {reference: "patient"}
          ]
        end

        if content_string
          payload[:payload] << content_string.fhir_payload
        end

        payload
      end

      def document_content
        if !attachment.nil?
          attachment.fhir_payload
        else
          binary.fhir_payload
        end
      end

      def load_communications
        json = application_client.get("/fhir/Communication?based-on=#{uuid}")

        @communications = Array(json["entry"]).map do |entry_json|
           Communication.new.attributes_from_json(entry_json["resource"])
        end

        @communications_loaded = true
      end

      def communications_loaded
        @communications_loaded ||= false
      end


  end
end
